@@ -22,17 +22,40 @@
 #include "favoritesmodel.h"
 #include "systementry.h"
 
+#include <QStandardPaths>
+
+#include <KDirWatch>
 #include <KLocalizedString>
 
 SystemModel::SystemModel(QObject *parent) : AbstractModel(parent)
-, m_favoritesModel(new FavoritesModel(this))
+{
+    init();
+
+    m_favoritesModel = new FavoritesModel(this);
+
+    const QString configFile = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + "/ksmserverrc";
+
+    KDirWatch *watch = new KDirWatch(this);
+
+    watch->addFile(configFile);
+
+    connect(watch, &KDirWatch::dirty, this, &SystemModel::refresh);
+    connect(watch, &KDirWatch::created, this, &SystemModel::refresh);
+}
+
+SystemModel::~SystemModel()
+{
+    qDeleteAll(m_entryList);
+}
+
+void SystemModel::init()
 {
     QList<SystemEntry *> actions;
 
     actions << new SystemEntry(this, SystemEntry::LockSession);
     actions << new SystemEntry(this, SystemEntry::LogoutSession);
     actions << new SystemEntry(this, SystemEntry::SaveSession);
-    actions << new SystemEntry(this, SystemEntry::NewSession);
+    actions << new SystemEntry(this, SystemEntry::SwitchUser);
     actions << new SystemEntry(this, SystemEntry::SuspendToRam);
     actions << new SystemEntry(this, SystemEntry::SuspendToDisk);
     actions << new SystemEntry(this, SystemEntry::Reboot);
@@ -47,11 +70,6 @@ SystemModel::SystemModel(QObject *parent) : AbstractModel(parent)
     }
 }
 
-SystemModel::~SystemModel()
-{
-    qDeleteAll(m_entryList);
-}
-
 QString SystemModel::description() const
 {
     return i18n("System actions");
@@ -69,6 +87,10 @@ QVariant SystemModel::data(const QModelIndex &index, int role) const
         return entry->name();
     } else if (role == Qt::DecorationRole) {
         return entry->iconName();
+    } else if (role == Kicker::DescriptionRole) {
+        return entry->description();
+    } else if (role == Kicker::GroupRole) {
+        return entry->group();
     } else if (role == Kicker::FavoriteIdRole) {
         return entry->id();
     }
@@ -95,7 +117,18 @@ bool SystemModel::trigger(int row, const QString &actionId, const QVariant &argu
     return false;
 }
 
-AbstractModel* SystemModel::favoritesModel()
+void SystemModel::refresh()
 {
-    return m_favoritesModel;
+    beginResetModel();
+
+    qDeleteAll(m_entryList);
+    m_entryList.clear();
+
+    init();
+
+    endResetModel();
+
+    emit countChanged();
+
+    m_favoritesModel->refresh();
 }
